/**
 * Common P2P Logging Classes
 * 
 * Copyright 2008, Yale Laboratory of Networked Systems
 */

package net.yale.cs.p4p.p2p.logs;

import java.util.Collections;
import java.util.List;

import net.yale.cs.p4p.p2p.logs.DownloadLogs.LogRecord;

public abstract class DownloadLogProcessor {

	private class BGThread extends Thread {
		
		private void doProcess(boolean terminating) {
			beforeProcess(terminating);

			/* Remove current records */
			List<LogRecord> records = getLogs().getRecords();

			/* Process records in this background thread */
			process(terminating, records);

			/* Push logs to next log processor in the chain */
			sendToNext(records);

			afterProcess(terminating);
		}
		
		public void run() {
			while (!interrupted()) {
				try { sleep(interval_); }
				catch (InterruptedException e) { break; }

				doProcess(false);
			}

			doProcess(true);
		}
	}

	/* Interval at which process() is called */
	private transient int interval_;
	protected void setInterval(int value) { interval_ = value; }
	public int getInterval() { return interval_; }

	/* Batched log records */
	private DownloadLogs logs_ = new DownloadLogs();
	protected DownloadLogs getLogs() { return logs_; }

	/* Next log processor in the chain */
	private DownloadLogProcessor next_;

	/* Background thread for log processing */
	private BGThread thread_ = new BGThread();

	public DownloadLogProcessor(DownloadLogProcessor next, int interval) {
		next_ = next;
		interval_ = interval;
		thread_.start();
	}

	public void stop() {
		thread_.interrupt();
		if (next_ != null)
			next_.stop();
	}

	public void join() {
		while (true) { try { thread_.join(); break; } catch (InterruptedException e) { } }
		if (next_ != null)
			next_.join();
	}

	/* Add single log record to the batch */
	public void push(LogRecord log) {
		push(Collections.singletonList(log));
	}

	/* Add collection of log records to the batch */
	public void push(List<LogRecord> logs) {
		logs_.addRecord(logs);
	}

	/* Send records to the next processor in the chain */
	protected final void sendToNext(List<LogRecord> logs) {
		if (next_ != null)
			next_.push(logs);
	}

	/* Abstract method for processing log records */
	protected abstract void process(boolean terminating, List<LogRecord> logs);

	/* Optional methods to be called before and after process calls */
	protected void beforeProcess(boolean terminating) {}
	protected void afterProcess(boolean terminating) {}

}
